@@ -1,7 +1,10 @@
+import json
 import urllib
+from datetime import datetime
+from time import sleep
 from meilisearch._httprequests import HttpRequests
 
-# pylint: disable=R0904
+# pylint: disable=too-many-public-methods
 class Index():
     """
     Indexes routes wrapper
@@ -81,13 +84,15 @@ class Index():
         return self.info()['primaryKey']
 
     @staticmethod
-    def create(config, **body):
+    def create(config, uid, options=None):
         """Create an index.
 
         Parameters
         ----------
-            body: **kwargs
-            Accepts uid, name and primaryKey as parameter.
+        uid: str
+            UID of the index
+        options: dict, optional
+            Options passed during index creation (ex: primaryKey)
 
         Returns
         -------
@@ -98,16 +103,9 @@ class Index():
         HTTPError
             In case of any error found here https://docs.meilisearch.com/references/#errors-status-code
         """
-        payload = {}
-        uid = body.get('uid', None)
-        if uid is not None:
-            payload['uid'] = uid
-        name = body.get('name', None)
-        if name is not None:
-            payload['name'] = name
-        primary_key = body.get('primary_key', None)
-        if primary_key is not None:
-            payload['primaryKey'] = primary_key
+        if options is None:
+            options = {}
+        payload = {'uid': uid, **options}
         return HttpRequests(config).post(config.paths.index, payload)
 
     @staticmethod
@@ -170,7 +168,7 @@ class Index():
         Returns
         ----------
         update: `list`
-            List of all enqueued and processed actions of the index.
+            List containing the details of the update status.
         """
         return self.http.get(
             '{}/{}/{}/{}'.format(
@@ -181,6 +179,33 @@ class Index():
             )
         )
 
+    def wait_for_pending_update(self, update_id, timeout_in_ms=5000, interval_in_ms=50):
+        """Wait until MeiliSearch processes an update, and get its status
+
+        Parameters
+        ----------
+        update_id: int
+            identifier of the update to retrieve
+        timeout_in_ms (optional): int
+            time the method should wait before rising a TimeoutError
+        interval_in_ms (optional): int
+            time interval the method should wait (sleep) between requests
+        Returns
+        ----------
+        update: `dict`
+            Dictionary containing the details of the processed update status.
+        """
+        start_time = datetime.now()
+        elapsed_time = 0
+        while elapsed_time < timeout_in_ms:
+            get_update = self.get_update_status(update_id)
+            if get_update['status'] != 'enqueued':
+                return get_update
+            sleep(interval_in_ms / 1000)
+            time_delta = datetime.now() - start_time
+            elapsed_time = time_delta.seconds * 1000 + time_delta.microseconds / 1000
+        raise TimeoutError
+
     def get_stats(self):
         """Get stats of an index
 
@@ -214,10 +239,18 @@ class Index():
         results: `dict`
             Dictionnary with hits, offset, limit, processingTime and initial query
         """
+        # Query parameters parsing
         if opt_params is None:
             opt_params = {}
-        search_param = {'q': query}
-        params = {**search_param, **opt_params}
+        for key in opt_params:
+            if key in ('facetsDistribution', 'facetFilters'):
+                opt_params[key] = json.dumps(opt_params[key])
+            elif isinstance(opt_params[key], list):
+                opt_params[key] = ','.join(opt_params[key])
+        params = {
+            'q': query,
+            **opt_params
+        }
         return self.http.get(
             '{}/{}/{}?{}'.format(
                 self.config.paths.index,
@@ -782,6 +815,54 @@ class Index():
             body
         )
 
+    # ATTRIBUTES FOR FACETING SUB-ROUTES
+
+    def get_attributes_for_faceting(self):
+        """
+        Get attributes for faceting of an index
+
+        Returns
+        ----------
+        settings: `list`
+            List containing the attributes for faceting of the index
+        """
+        return self.http.get(
+            self.__settings_url_for(self.config.paths.attributes_for_faceting)
+        )
+
+    def update_attributes_for_faceting(self, body):
+        """
+        Update attributes for faceting of an index
+
+        Parameters
+        ----------
+        body: `list`
+            List containing the attributes for faceting
+
+        Returns
+        ----------
+        update: `dict`
+            Dictionnary containing an update id to track the action:
+            https://docs.meilisearch.com/references/updates.html#get-an-update-status
+        """
+        return self.http.post(
+            self.__settings_url_for(self.config.paths.attributes_for_faceting),
+            body
+        )
+
+    def reset_attributes_for_faceting(self):
+        """Reset attributes for faceting of an index to default values
+
+        Returns
+        ----------
+        update: `dict`
+            Dictionnary containing an update id to track the action:
+            https://docs.meilisearch.com/references/updates.html#get-an-update-status
+        """
+        return self.http.delete(
+            self.__settings_url_for(self.config.paths.attributes_for_faceting),
+        )
+
     def __settings_url_for(self, sub_route):
         return '{}/{}/{}/{}'.format(
             self.config.paths.index,
